﻿using Connected.Utilities;
using Microsoft.AspNetCore.Components;

namespace Connected.Components;
public partial class DatePicker
{
	private bool loaded = false;

	private Selecting Selecting = Selecting.Days;
	private DateTime CalendarStart { get; set; } = DateTime.Today;
	private DateTime CalendarEnd { get; set; } = DateTime.Today;

	//private DateTime Today = DateTime.Today;

	[Parameter]
	public bool CloseOnDateSelect { get; set; } = false;

	[Parameter]
	public DateTime SelectedDate { get; set;  } = DateTime.Today;

	[Parameter]
	public DateTime? SelectedEndDate { get; set; } = null;

	[Parameter]
	public EventCallback<DateTime> SelectedDateChanged { get; set; }

	[Parameter]
	public EventCallback<DateTime?> SelectedEndDateChanged { get; set; }

	private List<DateTime> ShowingDates = new List<DateTime>();

	[Parameter]
	public bool UseDateRange { get; set; } = false;

	[Parameter]
	public string Format { get; set; } = "dd.MM.yyyy";

	private bool _readonly = false;
	[Parameter]
	public bool Editable
	{
		get
		{
			return !_readonly;
		}
		set
		{
			_readonly = !value;
		}
	}
	private bool Shown { get; set; } = false;

	public async Task SetDate(DateTime Date)
	{
		SelectedDate = Date;
		SetStartStop();
		if (CloseOnDateSelect)
			Shown = false;
		await SelectedDateChanged.InvokeAsync(SelectedDate);
		StateHasChanged();
	}

	public async Task SetMonth(int month)
	{
		SelectedDate = new DateTime(SelectedDate.Year, month, SelectedDate.Day);
		Selecting = Selecting.Days;
		SetStartStop();
		await SelectedDateChanged.InvokeAsync(SelectedDate);
		StateHasChanged();
	}

	public async Task SetYear(int year)
	{
		SelectedDate = new DateTime(year, SelectedDate.Month, SelectedDate.Day);
		Selecting = Selecting.Months;
		SetStartStop();
		await SelectedDateChanged.InvokeAsync(SelectedDate);
		StateHasChanged();
	}

	private string DayNamesRowClass
	{
		get
		{
			return new CssBuilder()
				.AddClass("chip-leading-icon")
				.AddClass("d-inline-block")
				.AddClass("m-1")
				.AddClass("bg-danger")
				.AddClass("text-small")
				.AddClass("text-white")
				.Build();
		}
	}
	private string ChipFirstRowStyle
	{
		get
		{
			return new StyleBuilder()
				.AddStyle("width", "30px")
				.AddStyle("height", "30px")
				.AddStyle("text-align", "center")
				.Build();
		}
	}
	/*
	 @for (int i=0;i<7; i++)
            {
                <div class="chip-leading-icon d-inline-block p-1 bg-danger text-small text-light" style="width:30px; height:30px; text-align:center">@calendarStart.AddDays(i).ToString("ddd").Substring(0,1)</div>
            }
            @while (calendarStart.CompareTo(calendarEnd) < 0)
            {
                <a href="#" @onclick="@(()=>SetDate(calendarStart))"><div class="chip-leading-icon d-inline-block p-1 @ChipStyle(calendarStart)" style="width:30px; height:30px; text-align:center">@calendarStart.Day.ToString()</div></a>
                @if (calendarStart.DayOfWeek.Equals(DayOfWeek.Sunday))
                {
                    <br />
                }
                calendarStart = calendarStart.AddDays(1);
            } 
	 
	 */

	public void SetStartStop()
	{
		CalendarStart = SelectedDate;
		while (CalendarStart.Day != 1)
		{
			CalendarStart = CalendarStart.AddDays(-1);
		}
		while (!CalendarStart.DayOfWeek.Equals(DayOfWeek.Monday))
		{
			CalendarStart = CalendarStart.AddDays(-1);
		}
		CalendarEnd = SelectedDate;
		while (CalendarEnd.Month == SelectedDate.Month)
		{
			CalendarEnd = CalendarEnd.AddDays(1);
		}
		while (!CalendarEnd.DayOfWeek.Equals(DayOfWeek.Monday))
		{
			CalendarEnd = CalendarEnd.AddDays(1);
		}
		DateTime start = CalendarStart;
		if (ShowingDates is null) ShowingDates = new();
		ShowingDates.Clear();
		while (start.CompareTo(CalendarEnd) < 0)
		{
			ShowingDates.Add(start);
			start = start.AddDays(1);
		}
	}

	public string DateChipStyle(DateTime date)
	{
		string result = "";
		if (date.Month.CompareTo(SelectedDate.Month) < 0 || date.Month.CompareTo(SelectedDate.Month) > 0) return "bg-core text-light";
		if (date.CompareTo(DateTime.Today) == 0) result = "b-1 b-c-info";
		if (date.Month.CompareTo(SelectedDate.Month) == 0)
		{
			if (date.Date.CompareTo(SelectedDate.Date) == 0)
				return result+" bg-info text-light";
			else
				return result+" bg-core text-dark";
		}
		return result;
	}

	public string MonthChipClass(int month)
	{
		if (month.CompareTo(SelectedDate.Month) == 0)
			return "bg-info text-light";
		else
			return "bg-core text-dark";
	}

	public string YearChipClass(int year)
	{
		if (year.CompareTo(SelectedDate.Year) == 0)
			return "bg-info text-light";
		else
			return "bg-core text-dark";
	}

	protected override async Task OnInitializedAsync()
	{
		SelectedDate = DateTime.Today;
		SetStartStop();
		loaded = true;
		await base.OnInitializedAsync();
	}

	private string NavBarClass
	{
		get
		{
			return new CssBuilder("text-justify")
				.AddClass("text-md-justify")
				//.AddClass("bg-warning")
				.AddClass("text-small")
				.AddClass("text-dark")
				.Build();
		}
	}

}

public enum Selecting
{
	Days,
	Months,
	Years
}
